//
//  MStringUtil.m
//  testapp
//

#include "MStringUtil.h"

namespace MStringUtil {

	static MString NSStringToMString(NSString *str)
	{
		const char *chars = [str cStringUsingEncoding:NSUTF8StringEncoding];
		return MString::fromUtf8String(chars);
	}

	static NSString *MStringToNSString(const MString& mstr)
	{
		return [[NSString alloc] initWithBytes:mstr.c_str() length:mstr.length()*sizeof(wchar_t) encoding:NSUTF32LittleEndianStringEncoding];
	}
	
	MAnsiString MStringToShiftJisString(const MString& mstr)
	{
		NSString *nsstr = [[NSString alloc] initWithBytes:mstr.c_str() length:mstr.length()*sizeof(wchar_t) encoding:NSUTF32LittleEndianStringEncoding];
		NSData* sjisData = [nsstr dataUsingEncoding:NSShiftJISStringEncoding allowLossyConversion:YES];
		const char *chars = static_cast<const char*>([sjisData bytes]);
		return MAnsiString(chars, (size_t)[sjisData length]);
	}

	MAnsiString MStringToUtf8String(const MString& mstr)
	{
		NSString *nsstr = [[NSString alloc] initWithBytes:mstr.c_str() length:mstr.length()*sizeof(wchar_t) encoding:NSUTF32LittleEndianStringEncoding];
		NSData* utf8Data = [nsstr dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES];
		const char *chars = static_cast<const char*>([utf8Data bytes]);
		return MAnsiString(chars, (size_t)[utf8Data length]);
	}
	
	MString ShiftJisStringToMString(const MAnsiString& ansistr)
	{
		if(ansistr.length() == 0)
			return MString();
		NSData *sjisData = [NSData dataWithBytes:ansistr.c_str() length:ansistr.length()+1];
		NSString *nsstr = [[NSString alloc] initWithData:sjisData encoding:NSShiftJISStringEncoding];
		const char *chars = [nsstr cStringUsingEncoding:NSUTF8StringEncoding];
		return MString::fromUtf8String(chars);
	}

	MString Utf8StringToMString(const MAnsiString& utf8str)
	{
		if(utf8str.length() == 0)
			return MString();
		NSData *utf8Data = [NSData dataWithBytes:utf8str.c_str() length:utf8str.length()+1];
		NSString *nsstr = [[NSString alloc] initWithData:utf8Data encoding:NSUTF8StringEncoding];
		const wchar_t *chars = (const wchar_t*)[nsstr cStringUsingEncoding:NSUTF32StringEncoding];
		return MString(chars);
	}
	
	MString ISOLatinStringToMString(const MAnsiString& isostr)
	{
		if(isostr.length() == 0)
			return MString();
		NSData *data = [NSData dataWithBytes:isostr.c_str() length:isostr.length()+1];
		NSString *nsstr = [[NSString alloc] initWithData:data encoding:NSISOLatin1StringEncoding];
		if(nsstr != nil){
			const wchar_t *chars = (const wchar_t*)[nsstr cStringUsingEncoding:NSUTF32StringEncoding];
			return MString(chars);
		}
		return MString();
	}

	MString GetMyDocumentDir()
	{
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
		for(NSString *str in paths){
			return NSStringToMString(str);
		}
		return MString();
	}

	MString GetMyPictureDir()
	{
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSPicturesDirectory, NSUserDomainMask, YES);
		for(NSString *str in paths){
			return NSStringToMString(str);
		}
		return MString();
	}
	
	MString GetDesktopDir()
	{
		NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDesktopDirectory, NSUserDomainMask, YES);
		for(NSString *str in paths){
			return NSStringToMString(str);
		}
		return MString();
	}
	
	MString ConvertToFullPath(const MString& relative_path)
	{
		return NSStringToMString([MStringToNSString(relative_path) stringByStandardizingPath]);
	}
	
	BOOL CreateDirectory(const MString& path)
	{
		return [[NSFileManager defaultManager] createDirectoryAtPath:MStringToNSString(path) withIntermediateDirectories:YES attributes:nil error:nil];
	}
	
	BOOL CopyFile(const MString& dst, const MString& src)
	{
		return [[NSFileManager defaultManager] copyItemAtPath:MStringToNSString(src) toPath:MStringToNSString(dst) error:nil];
	}
	
	BOOL IsHiddenFile(const MString& path, bool is_directory)
	{
		CFURLRef url = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, (CFStringRef)MStringToNSString(path), kCFURLPOSIXPathStyle, is_directory);
		LSItemInfoRecord itemInfo;
		LSCopyItemInfoForURL(url, kLSRequestAllFlags, &itemInfo);
		CFRelease(url);
		
		BOOL isInvisible = itemInfo.flags & kLSItemInfoIsInvisible;
		return isInvisible;
	}
	
}

