﻿//---------------------------------------------------------------------------
//
//   MQMorphManager.cpp      Copyright(C) 1999-2023, tetraface Inc.
//
//		An accessor for morph information
//
//    　モーフ情報のアクセス
//
//---------------------------------------------------------------------------

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include "MQMorphManager.h"

static const DWORD morph_plugin_product = 0x56A31D20;
static const DWORD morph_plugin_id = 0xC452C6DB;


MQMorphManager::MQMorphManager(MQBasePlugin *plugin, MQDocument doc)
	: m_Plugin(plugin), m_Doc(doc)
{
	int version = 0x4700;

	void *array[3];
	array[0] = (void*)"version";
	array[1] = &version;
	array[2] = nullptr;
	int ret = m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "QueryAPIVersion", array);
	m_Verified = (ret == 1);
}

DWORD MQMorphManager::GetProductID()
{
	return morph_plugin_product;
}

DWORD MQMorphManager::GetPluginID()
{
	return morph_plugin_id;
}

void MQMorphManager::BeginImport()
{
	m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "BeginImport", nullptr);
}

void MQMorphManager::EndImport()
{
	m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "EndImport", nullptr);
}

int MQMorphManager::GetBaseObjectNum()
{
	if(!m_Verified) return 0;

	int morph_num = m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "GetBaseObject", nullptr);
	return morph_num;
}

int MQMorphManager::EnumBaseObjects(std::vector<MQObject>& bases)
{
	int morph_num = GetBaseObjectNum();
	if(morph_num == 0){
		bases.clear();
		return 0;
	}
	bases.resize(morph_num);
	return m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "GetBaseObject", bases.data());
}

int MQMorphManager::GetTargetObjects(MQObject base, std::vector<MQObject>& targets)
{
	if(!m_Verified){
		targets.clear();
		return 0;
	}

	void *array[5];
	array[0] = (void*)"base";
	array[1] = base;
	array[2] = (void*)"target";
	array[3] = nullptr;
	array[4] = nullptr;

	int size = m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "GetTargetObject", array);
	
	if(size > 0){
		targets.resize(size);
		array[3] = targets.data();
		m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "GetTargetObject", array);
	}else{
		targets.clear();
	}
	return size;
}

bool MQMorphManager::GetTargetType(MQObject base, MQObject target, TargetType& type)
{
	if(!m_Verified) return false;

	void *array[7];
	array[0] = (void*)"base";
	array[1] = base;
	array[2] = (void*)"target";
	array[3] = target;
	array[4] = (void*)"type";
	array[5] = nullptr;
	array[6] = nullptr;

	int ret = m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "GetTargetType", array);
	if(ret && array[5] != nullptr){
		if(strcmp((char*)array[5], "brow") == 0)
			type = TARGET_BROW;
		else if(strcmp((char*)array[5], "eye") == 0)
			type = TARGET_EYE;
		else if(strcmp((char*)array[5], "lip") == 0)
			type = TARGET_LIP;
		else if(strcmp((char*)array[5], "other") == 0)
			type = TARGET_OTHER;
		else
			type = TARGET_DEFAULT;
		return true;
	}
	return false;
}

bool MQMorphManager::BindTargetObject(MQObject base, MQObject target)
{
	if(!m_Verified) return false;

	void *array[5];
	array[0] = (void*)"base";
	array[1] = base;
	array[2] = (void*)"target";
	array[3] = target;
	array[4] = nullptr;

	int ret = m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "BindTargetObject", array);
	return (ret != 0);
}

bool MQMorphManager::SetTargetType(MQObject base, MQObject target, TargetType type)
{
	if(!m_Verified) return false;

	void *array[7];
	array[0] = (void*)"base";
	array[1] = base;
	array[2] = (void*)"target";
	array[3] = target;
	array[4] = (void*)"type";
	array[5] = &type;
	array[6] = nullptr;

	int ret = m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "SetTargetType", array);
	return (ret != 0);
}

bool MQMorphManager::ApplyDeformation(MQObject base, MQObject proxy)
{
	if(!m_Verified) return false;

	void *array[5];
	array[0] = (void*)"base";
	array[1] = base;
	array[2] = (void*)"proxy";
	array[3] = proxy;
	array[4] = nullptr;
	return m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "ApplyDeformation", array) != 0;
}

bool MQMorphManager::SetPMDMode(bool flag)
{
	if(!m_Verified) return false;

	void *array[3];
	array[0] = (void*)"pmd";
	array[1] = &flag;
	array[2] = nullptr;

	int ret = m_Plugin->SendUserMessage(m_Doc, morph_plugin_product, morph_plugin_id, "SetPMDMode", array);
	return (ret != 0);
}

